SET CURRENT SCHEMA $RISCHK;


CREATE TABLE SC_SESSION(
		SESSION_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Session Name Missing' NOT NULL,
		BALLOT_DEFINITION_ID INTEGER DEFAULT -1 NOT NULL,
 		TEAM_ID INTEGER DEFAULT -1 NOT NULL,
 		PROJECT_ID INTEGER DEFAULT -1 NOT NULL,
		STATUS VARCHAR(255), 
 		SESSION_DATE TIMESTAMP, 
 		BALLOT_COUNT INTEGER DEFAULT 0 NOT NULL,
 		BALLOT_SUBMITTED INTEGER DEFAULT 0 NOT NULL,
 		"DESCRIPTION" VARCHAR(1000),
		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);

CREATE TABLE  SCSESSION_PARTICIPANT_LOOKUP (
		SESSION_ID INTEGER NOT NULL,
		PARTICIPANT_ID INTEGER NOT NULL
);




CREATE TABLE  SCSESSION_COFACILIT_LOOKUP (
		SESSION_ID INTEGER NOT NULL,
		FACILITATOR_ID INTEGER NOT NULL
);


CREATE TABLE BALLOT_DEFINITION(
		BALLOT_DEFINITION_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Ballot Definition Name Missing' NOT NULL,
    	SCALE_ID INTEGER DEFAULT -1 NOT NULL,
    	"DESCRIPTION" VARCHAR(1000),
		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);    
    

CREATE TABLE  BALLOTDEF_BALLOT_TAB_LOOKUP (
		BALLOT_DEFINITION_ID INTEGER NOT NULL,
		BALLOT_TAB_ID INTEGER NOT NULL
);  
  
 
CREATE TABLE BALLOT(
		BALLOT_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Ballot Name Missing' NOT NULL,
		SESSION_ID INTEGER DEFAULT -1 NOT NULL,
 		STATUS VARCHAR(25),
		SUBMISSIONTIME TIMESTAMP,
		SUBMITTER_ID INTEGER DEFAULT -1 NOT NULL,
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);    
		
CREATE TABLE ANSWER(
		ANSWER_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Answer Name Missing' NOT NULL,
 		QUESTION_ID INTEGER DEFAULT -1 NOT NULL,
 		BALLOT_ID INTEGER DEFAULT -1 NOT NULL,
 		BALLOT_TAB_ID INTEGER DEFAULT -1 NOT NULL,
 		VALUE VARCHAR(255),
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);   			


CREATE TABLE BALLOT_TAB(
		BALLOT_TAB_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Ballot Tab Name Missing' NOT NULL,
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);   		



CREATE TABLE QUESTION(
		QUESTION_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Question Name Missing' NOT NULL,
 		SCALE_ID INTEGER DEFAULT -1 NOT NULL,
 		"DESCRIPTION" VARCHAR(1000),
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);  					




CREATE TABLE BALLOT_TAB_QUESTION_LOOKUP(
		BALLOT_TAB_ID INTEGER NOT NULL,
		QUESTION_ID INTEGER NOT NULL,
		QUESTION_ORDER INTEGER
); 

	

CREATE TABLE SC_SCALE(
		SCALE_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Scale Name Missing' NOT NULL,
 		TRUE_VALUE_DISPLAY VARCHAR(255),
 		FALSE_VALUE_DISPLAY VARCHAR(255),
 		INT_START_VALUE INTEGER,
 		INT_END_VALUE INTEGER,
 		DEC_START_VALUE DECIMAL(18,2),
 		DEC_END_VALUE DECIMAL(18,2),
 		PRECISION_VALUE INTEGER,
 		TALKTHRESHOLDAVG DOUBLE,
 		TALKTHRESHOLDDEV DOUBLE,
 		TYPE VARCHAR(255),
 		LENGTH INTEGER,
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
);  		




CREATE TABLE SC_BALLOT_TAB_RESULT (
 		RESULT_ID INTEGER NOT NULL,
		REFERENCE_ID VARCHAR(50),
		EXTERNAL_ID VARCHAR(100),
		NAME VARCHAR(255) DEFAULT 'Result Name Missing' NOT NULL,
 		SESSION_ID INTEGER DEFAULT -1 NOT NULL,
 		BALLOT_TAB_ID INTEGER DEFAULT -1 NOT NULL,
 		STANDARD_DEV_VALUE DECIMAL (18,2),
 		AVERAGE_VALUE DECIMAL (18,2),
 		EXTERNAL_KEY1 INTEGER DEFAULT -1 NOT NULL,
		EXTERNAL_KEY2 CHAR(40) DEFAULT '-1' NOT NULL,
		SOURCE_ID VARCHAR(36) DEFAULT '-1' NOT NULL,
		URL VARCHAR(2048),
		REC_DATETIME TIMESTAMP DEFAULT CURRENT TIMESTAMP NOT NULL,
		ISSOFTDELETED SMALLINT DEFAULT 0 NOT NULL
); 

-- INDEXES

CREATE INDEX ANSWER_IDX3 ON ANSWER (EXTERNAL_KEY2) ;                                             
CREATE INDEX BALTAB_IDX3 ON BALLOT_TAB (EXTERNAL_KEY2) ;                                        
CREATE INDEX QUESTION_IDX3 ON QUESTION (EXTERNAL_KEY2) ;                                          
CREATE INDEX SCRESULT_IDX3 ON SC_BALLOT_TAB_RESULT (EXTERNAL_KEY2) ;                                       
CREATE INDEX SCSCALE_IDX3 ON SC_SCALE (EXTERNAL_KEY2) ;                                         
CREATE INDEX SCSESSION_IDX3 ON SC_SESSION (EXTERNAL_KEY2) ;    

CREATE INDEX ANSWER_IDX4 ON ANSWER (EXTERNAL_KEY1,SOURCE_ID) ;                                             
CREATE INDEX BALTAB_IDX4 ON BALLOT_TAB (EXTERNAL_KEY1,SOURCE_ID) ;                                     
CREATE INDEX QUESTION_IDX4 ON QUESTION (EXTERNAL_KEY1,SOURCE_ID) ;                                         
CREATE INDEX SCRESULT_IDX4 ON SC_BALLOT_TAB_RESULT (EXTERNAL_KEY1,SOURCE_ID) ;                                       
CREATE INDEX SCSCALE_IDX4 ON SC_SCALE (EXTERNAL_KEY1,SOURCE_ID) ;                                         
CREATE INDEX SCSESSION_IDX4 ON SC_SESSION (EXTERNAL_KEY1,SOURCE_ID) ;                                      
 
CREATE INDEX ANSWER_IDX5 ON ANSWER (EXTERNAL_KEY2,SOURCE_ID) ;                                             
CREATE INDEX BALTAB_IDX5 ON BALLOT_TAB (EXTERNAL_KEY2,SOURCE_ID) ;                                     
CREATE INDEX QUESTION_IDX5 ON QUESTION (EXTERNAL_KEY2,SOURCE_ID) ;                                         
CREATE INDEX SCRESULT_IDX5 ON SC_BALLOT_TAB_RESULT (EXTERNAL_KEY2,SOURCE_ID) ;                                       
CREATE INDEX SCSCALE_IDX5 ON SC_SCALE (EXTERNAL_KEY2,SOURCE_ID) ;                                         
CREATE INDEX SCSESSION_IDX5 ON SC_SESSION (EXTERNAL_KEY2,SOURCE_ID) ; 
 
CREATE INDEX ANSWER_IDX2 ON ANSWER (EXTERNAL_ID) ;        
CREATE INDEX BALTAB_IDX2 ON BALLOT_TAB (EXTERNAL_ID) ;    
CREATE INDEX QUESTION_IDX2 ON QUESTION (EXTERNAL_ID) ;    
CREATE INDEX SCRESULT_IDX2 ON SC_BALLOT_TAB_RESULT (EXTERNAL_ID) ;   
CREATE INDEX SCSCALE_IDX2 ON SC_SCALE (EXTERNAL_ID) ;     
CREATE INDEX SCSESSION_IDX2 ON SC_SESSION (EXTERNAL_ID) ; 
 
 
-- PRIMARY KEYS

ALTER TABLE SC_SESSION ADD CONSTRAINT SCSESS_PK PRIMARY KEY (SESSION_ID);

ALTER TABLE SCSESSION_PARTICIPANT_LOOKUP ADD CONSTRAINT SCSESSPART_PK PRIMARY KEY (SESSION_ID,PARTICIPANT_ID);

ALTER TABLE BALLOT_DEFINITION ADD CONSTRAINT BALLOTDEF_PK PRIMARY KEY (BALLOT_DEFINITION_ID);   

ALTER TABLE BALLOTDEF_BALLOT_TAB_LOOKUP ADD CONSTRAINT BALDEFTAB_PK PRIMARY KEY (BALLOT_DEFINITION_ID,BALLOT_TAB_ID); 

ALTER TABLE BALLOT ADD CONSTRAINT BALLOT_PK PRIMARY KEY (BALLOT_ID); 

ALTER TABLE ANSWER ADD CONSTRAINT ANSWER_PK PRIMARY KEY (ANSWER_ID); 

ALTER TABLE BALLOT_TAB ADD CONSTRAINT BALLOT_TAB_PK PRIMARY KEY (BALLOT_TAB_ID); 

ALTER TABLE QUESTION ADD CONSTRAINT QUESTION_PK PRIMARY KEY (QUESTION_ID); 

ALTER TABLE BALLOT_TAB_QUESTION_LOOKUP ADD CONSTRAINT BALTBQUELK_PK PRIMARY KEY (BALLOT_TAB_ID,QUESTION_ID); 

ALTER TABLE SCSESSION_COFACILIT_LOOKUP ADD CONSTRAINT SCSESSFASC_PK PRIMARY KEY (SESSION_ID,FACILITATOR_ID);	

ALTER TABLE SC_SCALE ADD CONSTRAINT SCSCALE_PK PRIMARY KEY (SCALE_ID); 

ALTER TABLE SC_BALLOT_TAB_RESULT ADD CONSTRAINT SCRESULT_PK PRIMARY KEY (RESULT_ID); 

-- FOREIGN KEYS

ALTER TABLE SC_SESSION ADD CONSTRAINT SCSESS_FK1 FOREIGN KEY (PROJECT_ID)
		REFERENCES $NF_SCHEMA.PROJECT (PROJECT_ID);
		
ALTER TABLE SC_SESSION ADD CONSTRAINT SCSESS_FK2 FOREIGN KEY (TEAM_ID)
		REFERENCES $NF_SCHEMA.TEAM (TEAM_ID);		
		
ALTER TABLE SC_SESSION ADD CONSTRAINT SCSESS_FK3 FOREIGN KEY (BALLOT_DEFINITION_ID)
		REFERENCES  BALLOT_DEFINITION (BALLOT_DEFINITION_ID);							

ALTER TABLE SCSESSION_PARTICIPANT_LOOKUP ADD CONSTRAINT SCSESSPART_FK1 FOREIGN KEY (SESSION_ID)
		REFERENCES  SC_SESSION (SESSION_ID);
		
ALTER TABLE SCSESSION_PARTICIPANT_LOOKUP ADD CONSTRAINT SCSESSPART_FK2 FOREIGN KEY (PARTICIPANT_ID)
		REFERENCES  $NF_SCHEMA.RESOURCE (RESOURCE_ID);			

ALTER TABLE SCSESSION_COFACILIT_LOOKUP ADD CONSTRAINT SCSESSFASC_FK1 FOREIGN KEY (SESSION_ID)
		REFERENCES  SC_SESSION (SESSION_ID);

ALTER TABLE SCSESSION_COFACILIT_LOOKUP ADD CONSTRAINT SCSESSFASC_FK2 FOREIGN KEY (FACILITATOR_ID)
		REFERENCES  $NF_SCHEMA.RESOURCE (RESOURCE_ID);	
    
ALTER TABLE BALLOT_DEFINITION ADD CONSTRAINT BALLOTDEF_FK3 FOREIGN KEY (SCALE_ID)
		REFERENCES SC_SCALE (SCALE_ID);		     

ALTER TABLE BALLOTDEF_BALLOT_TAB_LOOKUP ADD CONSTRAINT BALDEFTAB_FK1 FOREIGN KEY (BALLOT_DEFINITION_ID)
		REFERENCES  BALLOT_DEFINITION(BALLOT_DEFINITION_ID);

ALTER TABLE BALLOTDEF_BALLOT_TAB_LOOKUP ADD CONSTRAINT BALDEFTAB_FK2 FOREIGN KEY (BALLOT_TAB_ID)
		REFERENCES  BALLOT_TAB (BALLOT_TAB_ID);		

ALTER TABLE ANSWER ADD CONSTRAINT ANSWER_FK1 FOREIGN KEY (QUESTION_ID)
		REFERENCES QUESTION (QUESTION_ID);	

ALTER TABLE ANSWER ADD CONSTRAINT ANSWER_FK2 FOREIGN KEY (BALLOT_ID)
		REFERENCES BALLOT (BALLOT_ID);
		
ALTER TABLE ANSWER ADD CONSTRAINT ANSWER_FK3 FOREIGN KEY (BALLOT_TAB_ID)
		REFERENCES BALLOT_TAB (BALLOT_TAB_ID);		
						
ALTER TABLE QUESTION ADD CONSTRAINT QUESTION_FK3 FOREIGN KEY (SCALE_ID)
		REFERENCES SC_SCALE (SCALE_ID);		
		 
ALTER TABLE BALLOT_TAB_QUESTION_LOOKUP ADD CONSTRAINT BALTBQUELK_FK1 FOREIGN KEY (BALLOT_TAB_ID)
		REFERENCES BALLOT_TAB (BALLOT_TAB_ID);		
		
ALTER TABLE BALLOT_TAB_QUESTION_LOOKUP ADD CONSTRAINT BALTBQUELK_FK2 FOREIGN KEY (QUESTION_ID)
		REFERENCES QUESTION (QUESTION_ID);	
		 		
ALTER TABLE SC_BALLOT_TAB_RESULT ADD CONSTRAINT SCRESULT_FK3 FOREIGN KEY (SESSION_ID)
		REFERENCES SC_SESSION (SESSION_ID);
		
ALTER TABLE SC_BALLOT_TAB_RESULT ADD CONSTRAINT SCRESULT_FK4 FOREIGN KEY (BALLOT_TAB_ID)
		REFERENCES BALLOT_TAB (BALLOT_TAB_ID);			

--TRIGGERS

CREATE TRIGGER TRG_BALDEFBALTAB
	AFTER UPDATE OF REC_DATETIME ON BALLOT_DEFINITION
	REFERENCING  NEW AS NEWROW                                                           
		OLD AS OLDROW                                                                      
	FOR EACH ROW MODE DB2SQL                                                             
BEGIN ATOMIC                                                                           
    DELETE FROM BALLOTDEF_BALLOT_TAB_LOOKUP WHERE BALLOT_DEFINITION_ID=NEWROW.BALLOT_DEFINITION_ID; END;
     
CREATE TRIGGER TRG_BALTABQUES
	AFTER UPDATE OF REC_DATETIME ON QUESTION
	REFERENCING  NEW AS NEWROW                                                           
		OLD AS OLDROW                                                                      
	FOR EACH ROW MODE DB2SQL                                                             
BEGIN ATOMIC                                                                           
    DELETE FROM BALLOT_TAB_QUESTION_LOOKUP WHERE QUESTION_ID=NEWROW.QUESTION_ID; END;    
    
CREATE TRIGGER TRG_SESCOFACILI
	AFTER UPDATE OF REC_DATETIME ON SC_SESSION
	REFERENCING  NEW AS NEWROW                                                           
		OLD AS OLDROW                                                                      
	FOR EACH ROW MODE DB2SQL                                                             
BEGIN ATOMIC                                                                           
    DELETE FROM SCSESSION_COFACILIT_LOOKUP WHERE SESSION_ID=NEWROW.SESSION_ID; END;    
        
CREATE TRIGGER TRG_SESPARTICI
	AFTER UPDATE OF REC_DATETIME ON SC_SESSION
	REFERENCING  NEW AS NEWROW                                                           
		OLD AS OLDROW                                                                      
	FOR EACH ROW MODE DB2SQL                                                             
BEGIN ATOMIC                                                                           
    DELETE FROM SCSESSION_PARTICIPANT_LOOKUP WHERE SESSION_ID=NEWROW.SESSION_ID; END;  
    
COMMIT;